----------------------------------------------------------------------------------
-- Company: Kyvox Technologies
-- Author: LEVESQUE Philippe
-- 
-- Create Date:    14:07:58 04/22/2009 
-- Design Name: 
-- Module Name:    antirebond_1b - Behavioral
-- Project Name: 
-- Target Devices: 
-- Tool versions: 
-- Description: Use a 8bit shift register as a filter for switch
--              number of switch is defined by GENERIC_NUMBER_SWITCH
--              logic level is set by GENERIC_SW_LOGIC and is used to set initial value
--
-- Dependencies: None
--
-- Revision: 
-- Revision 0.01 - File Created
-- Additional Comments: 
--                       for 1ms debouncing, clock of 8kHz should be used
--                       for 64us debouncing, clock of 125kHz should be used
--                       for 1us debouncing, clock of 8MHz should be used
----------------------------------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.Numeric_std.ALL;

entity antirebond_1b is	
   generic
        (
        GENERIC_SW_LOGIC : std_logic := '1'-- 1=POSITIVE 0=NEGATIVE
        );
    Port ( 
        rst_i : in std_logic;
        clk_i : in std_logic;
        b_i : in std_logic;
        b_o: out std_logic
    );
end antirebond_1b;


architecture arch_antirebond_1b of antirebond_1b is

signal b_shiftreg_p, b_shiftreg_f : std_logic_vector (7 downto 0);
signal b_clean_p, b_clean_f : std_logic;

begin

b_o <= b_clean_p ;

P_sync: process(clk_i, rst_i)
begin
     if rst_i='1' then
        b_shiftreg_p <= (others=>(not(GENERIC_SW_LOGIC)));
        b_clean_p <= not(GENERIC_SW_LOGIC);
     elsif clk_i'event and clk_i='1' then
         b_shiftreg_p <= b_shiftreg_f;
         b_clean_p <= b_clean_f;
     end if;
end process;

b_shiftreg_f <= b_shiftreg_p(6 downto 0) & b_i;
	
P_comb: process(b_shiftreg_p, b_clean_p)
begin 
    if b_shiftreg_p = "11111111" then
	    b_clean_f <= '1';
    elsif b_shiftreg_p = "00000000" then
         b_clean_f <= '0';
    else
         b_clean_f <= b_clean_p;
    end if;
end process;


end arch_antirebond_1b;
