----------------------------------------------------------------------------------
-- Author: LEVESQUE Philippe
-- 
-- Create Date:    15:52:55 01/13/2012 
-- Design Name: 
-- Module Name:    sync_io - Behavioral 
-- Project Name: 
-- Target Devices: 
-- Tool versions: 
-- Description: synchronisation module
--              number of inputs is defined by GENERIC_NUMBER_IO 
--              logic level is set by GENERIC_IO_LOGIC and is used to set initial value 
--					 (same logic for all IOs)
--
-- Dependencies: None
--
-- Revision: 
-- Revision 0.01 - File Created
-- Additional Comments: 
--                       
--                       
----------------------------------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.NUMERIC_STD.ALL;



entity sync_io_1 is	
   generic
        (
        GENERIC_IO_LOGIC : std_logic := '1'-- 1=POSITIVE 0=NEGATIVE
        );
    Port ( 
        rst_i : in std_logic;
        --front-end
        clk_front_end_i : in std_logic;
        io_i : in std_logic;
        --back-end
        clk_back_end_i : in std_logic;
        io_o : out std_logic
    );
end sync_io_1;

architecture arch_sync_io_1 of sync_io_1 is


signal io_shiftreg_p, io_shiftreg_f : std_logic_vector (2 downto 0);
signal io_in_p, io_in_f : std_logic;


begin

io_o <= io_shiftreg_p(2);

front_end_process: process(clk_front_end_i, rst_i)
    begin
         if rst_i='1' then
	          io_in_p <= not(GENERIC_IO_LOGIC);
	    elsif clk_front_end_i'event and clk_front_end_i='1' then
	          io_in_p <= io_in_f;  
	    end if;
	end process;

io_in_f <= io_i;


back_end_process: process(clk_back_end_i, rst_i)
    begin
	     if rst_i='1' then
	         io_shiftreg_p <= (others=>(not(GENERIC_IO_LOGIC)));
	     elsif clk_back_end_i'event and clk_back_end_i='1' then
	         io_shiftreg_p <= io_shiftreg_f;
		end if;
	end process;

 io_shiftreg_f <= io_shiftreg_p(1 downto 0) & io_in_p;




end arch_sync_io_1;
