library IEEE;
use IEEE.STD_LOGIC_1164.ALL;


entity tp1_ex2_top is
  Port ( 
      rst_i_n : in std_logic;
      clk_100mhz_i : in std_logic;
      btnc : in std_logic;
      btnd: in std_logic;
      btnl : in std_logic;
      btnr: in std_logic;
      btnu : in std_logic;
      dels_o: out std_logic_vector(7 downto 0)
  );
end tp1_ex2_top;

architecture Behavioral of tp1_ex2_top is
component antirebond_1b
port
  (
  rst_i : in std_logic;
  clk_i : in std_logic;
  b_i : in std_logic;
  b_o : out std_logic
  );
end component; 
component sync_io_1
generic
  (
  GENERIC_IO_LOGIC : std_logic :='1'-- 1=POSITIVE 0=NEGATIVE
  );
port
  (
  rst_i : in std_logic;
  --front-end
  clk_front_end_i : in std_logic;
  io_i : in std_logic;
  --back-end
  clk_back_end_i	: in std_logic;
  io_o : out std_logic
  );
end component;

component clk_wiz
port
 (-- Clock in ports
  clk_in1           : in     std_logic;
  -- Clock out ports
  clk_out1          : out    std_logic;
  -- Status and control signals
  reset             : in     std_logic;
  locked            : out    std_logic
 );
end component;
	
component divise_clk_2000
port
  ( 
  rst_i : in  STD_LOGIC; 
  clk_i : in  STD_LOGIC; 
  clk_o : out  STD_LOGIC
  );
end component;

component msa_tp1_ex2 
port
  (
  clk_i : in  STD_LOGIC;
  rst_i : in  STD_LOGIC;
  bouton_left_i : in  STD_LOGIC;
  bouton_up_i : in  STD_LOGIC;
  bouton_right_i : in STD_LOGIC;
  bouton_down_i : in STD_LOGIC;
  bouton_center_i : in  STD_LOGIC;
  dels_o : out std_logic_vector(7 downto 0));
end component;

signal rst_i : std_logic;

signal clk_16mhz : std_logic;
signal locked : std_logic;
signal rst_sys : std_logic;
signal clk8khz : std_logic;

signal bouton_ar_left : std_logic;
signal bouton_ar_up : std_logic;
signal bouton_ar_center : std_logic;
signal bouton_ar_right : std_logic;
signal bouton_ar_down : std_logic;

signal bouton_sync_left : std_logic;
signal bouton_sync_up : std_logic;
signal bouton_sync_center : std_logic;
signal bouton_sync_right : std_logic;
signal bouton_sync_down : std_logic;

begin

rst_i <= not rst_i_n;

inst_deb_left: antirebond_1b 
port map
  (
  rst_i => rst_sys,
  b_i => btnl,
  b_o => bouton_ar_left, 
  clk_i => clk8khz
);

inst_deb_up: antirebond_1b 
port map
  (
  rst_i => rst_sys,
  b_i => btnu,
  b_o => bouton_ar_up, 
  clk_i => clk8khz
);

inst_deb_center: antirebond_1b 
port map
  (
  rst_i => rst_sys,
  b_i => btnc,
  b_o => bouton_ar_center, 
  clk_i => clk8khz
);

inst_deb_right: antirebond_1b 
port map
  (
  rst_i => rst_sys,
  b_i => btnr,
  b_o => bouton_ar_right, 
  clk_i => clk8khz
);

inst_deb_down: antirebond_1b 
port map
  (
  rst_i => rst_sys,
  b_i => btnd,
  b_o => bouton_ar_down, 
  clk_i => clk8khz
);

Inst_sync_io_left: sync_io_1
    generic map
      (
      GENERIC_IO_LOGIC => '1' -- 1=POSITIVE 0=NEGATIVE
      )
    port map
      (
      rst_i => rst_sys,
      --front-end
      clk_front_end_i => clk8khz,
      io_i => bouton_ar_left,
      --back-end
      clk_back_end_i	=>  clk_16mhz,
      io_o => bouton_sync_left
      );
  
Inst_sync_io_up: sync_io_1
  generic map
    (
    GENERIC_IO_LOGIC => '1' -- 1=POSITIVE 0=NEGATIVE
    )
  port map
    (
    rst_i => rst_sys,
    --front-end
    clk_front_end_i => clk8khz,
    io_i => bouton_ar_up,
    --back-end
    clk_back_end_i    =>  clk_16mhz,
    io_o => bouton_sync_up
    );
    
Inst_sync_io_center: sync_io_1
    generic map
      (
      GENERIC_IO_LOGIC => '1' -- 1=POSITIVE 0=NEGATIVE
      )
    port map
      (
      rst_i => rst_sys,
      --front-end
      clk_front_end_i => clk8khz,
      io_i => bouton_ar_center,
      --back-end
      clk_back_end_i    =>  clk_16mhz,
      io_o => bouton_sync_center
      );
	
Inst_sync_io_right: sync_io_1
      generic map
        (
        GENERIC_IO_LOGIC => '1' -- 1=POSITIVE 0=NEGATIVE
        )
      port map
        (
        rst_i => rst_sys,
        --front-end
        clk_front_end_i => clk8khz,
        io_i => bouton_ar_right,
        --back-end
        clk_back_end_i    =>  clk_16mhz,
        io_o => bouton_sync_right
        );
        
Inst_sync_io_down: sync_io_1
      generic map
        (
        GENERIC_IO_LOGIC => '1' -- 1=POSITIVE 0=NEGATIVE
        )
      port map
        (
        rst_i => rst_sys,
        --front-end
        clk_front_end_i => clk8khz,
        io_i => bouton_ar_down,
        --back-end
        clk_back_end_i    =>  clk_16mhz,
        io_o => bouton_sync_down
        );
        
Inst_clk_wiz: clk_wiz
port map
  (     
  -- Clock in ports
  clk_in1 => clk_100mhz_i,
  -- Clock out ports  
  clk_out1 => clk_16mhz,
  -- Status and control signals                
  reset => rst_i,
  locked => locked            
  );
	
rst_sys <= not(locked);

inst_divise_clk: divise_clk_2000
port map
  (
  rst_i => rst_sys,
  clk_i => clk_16mhz,
  clk_o => clk8khz
  );
 
Inst_msa: msa_tp1_ex2 
port map
  (
  clk_i => clk_16mhz,
  rst_i => rst_sys,
  bouton_left_i => bouton_sync_left,
  bouton_up_i => bouton_sync_up,
  bouton_center_i => bouton_sync_center,
  bouton_right_i => bouton_sync_right,
  bouton_down_i => bouton_sync_down,
  dels_o => dels_o --directement relie aux DELs
  ); 
end Behavioral;
